import { Component, OnDestroy, OnInit, ChangeDetectorRef, ViewChild, AfterViewInit } from '@angular/core';
import { DeploymentNotificationService } from '../services/deployment-notification.service';
import { Subscription } from 'rxjs/Subscription';
import { WebSocketService } from '../services/websocket.service';
import { LayoutCommand } from './layout-command';
import { config } from '../config';
import { SidebarComponent } from '../sidebar/sidebar.component';

@Component({
  selector: 'app-dashboard',
  templateUrl: './dashboard.component.html',
  styleUrls: ['./dashboard.component.scss']
})
export class DashboardComponent implements OnInit, OnDestroy, AfterViewInit {
  deploying = false;
  deploymentNotificationSubscription: Subscription;
  enabledLayouts: Set<string> = config.dashboard.enabledLayouts;
  layout: string = config.dashboard.currentLayout ;
  command_handlers = {'layout': this.layoutHandler.bind(this)};

  @ViewChild(SidebarComponent)
  private sidebarComponent: SidebarComponent;

  constructor(private deploymentNotificationService: DeploymentNotificationService,
              private webSocketService: WebSocketService,
              private changeDetectorRef: ChangeDetectorRef) {}

  ngOnInit() {
    this.webSocketService.connect();
    this.webSocketService.observeKey<LayoutCommand>('dashboard').subscribe((event) => {
      this.command_handlers[event.data.command](event.data);
      this.changeDetectorRef.detectChanges();
    });
    this.deploymentNotificationSubscription = this.deploymentNotificationService.deploying.subscribe(
      (deploying) => this.deploying = deploying
    );
  }

  ngAfterViewInit() {
    this.layout = this.sidebarComponent.layout;
  }

  layoutHandler(data: LayoutCommand) {
    if (this.enabledLayouts.has(data.layout)) {
      this.layout = data.layout;
    }
    else {
      console.log('Invalid ide layout "' + data.layout + '" received!');
    }
  }

  ngOnDestroy() {
    if (this.deploymentNotificationSubscription) {
      this.deploymentNotificationSubscription.unsubscribe();
    }
  }
}
